from vpython import *
#GlowScript 3.0 VPython
## three dimensional simulation of the three-body problem

global threebody
scene = display(title='University of Glasgow A1 Dynamical Astronomy orbit demos (press 1-7). RH mouse rotates, LH+RH zooms.', width=1880, height=950)


## set up the masses. m1 is missing in 2-body systems
m1 = sphere(pos=vec( 10,0,0),vel=vec( 0.0, 1.0,0.0),
              mass=10.0, color=color.red,
              make_trail=True, interval=2, retain=1000)
m2 = sphere(pos=vec(-10,0,3),vel=vec(-0.3,-1.0,0.5),
              mass=10.0, color=color.yellow,
              make_trail=True, interval=2, retain=1000)
m3 = sphere(pos=vec(  0,0,0),vel=vec(   0, 0.1,0.1),
              mass=10.0, color=color.white,
              make_trail=True, interval=2, retain=1000)
              
## move to the centre of mass frame and adjust sizes for constant density
def resetscene(m1,m2,m3):
    vcentre=(m1.mass*m1.vel+m2.mass*m2.vel+m3.mass*m3.vel)/(m1.mass+m2.mass+m3.mass)
    for a in [m1, m2, m3]:
        a.vel -= vcentre
        a.radius = 0.5*a.mass**(1.0/3.0)
        a.clear_trail() 

def dydt(y):
    deriv = [vec(0,0,0)] * 6
    r12 = y[0]-y[2] 
    r23 = y[2]-y[4] 
    r31 = y[4]-y[0]
    r12c = r12.hat/mag2(r12)
    r23c = r23.hat/mag2(r23)
    r31c = r31.hat/mag2(r31)
    deriv[1] = -m2.mass*r12c + m3.mass*r31c
    deriv[3] = -m3.mass*r23c + m1.mass*r12c
    deriv[5] = -m1.mass*r31c + m2.mass*r23c
    deriv[0] = y[1];  deriv[2] = y[3];  deriv[4] = y[5]
    return mult(dt, deriv)
    
def mult(f,v):
    c = [vec(0,0,0)] * 6
    for i in range(0,6):
     c[i] = f*v[i]
    return c
    
def add(j,k):
    c= [vec(0,0,0)] * 6
    for i in range(0,6):
     c[i] = j[i] + k[i]
    return c


def keyInput(evt):
    global threebody,dt
    s = evt.key
    if (s == '0'): ##circular
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
        m2.pos=vector( -10,0,0);   m2.vel=vector( 0, 0,0); m2.mass=10
        m3.pos=vector( 10,0,0);  m3.vel=vector(0,0,0);  m3.mass=5
        scene.range=15
        resetscene(m1,m2,m3)
        threebody = False;
    if (s == '1'): ##circular
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
        m2.pos=vector( 0,0,0);   m2.vel=vector( 0, 0,0); m2.mass=10
        m3.pos=vector( 10,0,0);  m3.vel=vector(0,-1,0);  m3.mass=0.1
        scene.range=20
        resetscene(m1,m2,m3)
        threebody = False;
    if (s == '2'):
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
        m2.pos=vector( 5,0,0);   m2.vel=vector(0,0,0);   m2.mass=10.0
        m3.pos=vector(-10,0,0);  m3.vel=vector(0,0.5,0); m3.mass=0.1
        scene.range=15
        resetscene(m1,m2,m3)
        threebody= False
    if (s == '3'):
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
        m2.pos=vector( 5,0,0);   m2.vel=vector(0,1.0/3.0,0);   m2.mass=10.0
        m3.pos=vector(-10,0,0);  m3.vel=vector(0,-2.0/3.0,0); m3.mass=5.0
        scene.range=20
        resetscene(m1,m2,m3)
        threebody= False
    if (s == '4'):
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);    m1.mass=0
        m2.pos=vector(-10,0,0);  m2.vel=vector(0,0.2,0);  m2.mass=5.0
        m3.pos=vector( 10,0,0);  m3.vel=vector(0,-0.2,0); m3.mass=5.0
        scene.range=15
        resetscene(m1,m2,m3)
        threebody= False
    if (s == '5'):
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);    m1.mass=0
        m2.pos=vector(-10,0,0);  m2.vel=vector(0,0.2,0);  m2.mass=10.0
        m3.pos=vector( 10,0,0);  m3.vel=vector(0,-0.2,0); m3.mass=5.0
        scene.range=15
        resetscene(m1,m2,m3)
        threebody= False
    if (s == '6'):
        threebody = True;
        m1.pos=vector( 10.5,0,0); m1.vel=vector(0.0, sqrt(10.0/10.5),0.0); m1.mass=0.01; m1.retain=500
        m2.pos=vector(0,0,0); m2.vel=vector(0,0,0);m2.mass=10.0
        m3.pos=vector(-10,0,0);  m3.vel=vector( 0, -1,0);  m3.mass=0.01;m3.retain=500
        scene.range=20
        #dt=0.1
        resetscene(m1,m2,m3)
        m1.radius = m1.radius*3;m3.radius = m3.radius*3
    if (s == '7'):
        threebody = True;
        m1.pos=vector( 10,0,0); m1.vel=vector(0.0, 1.0,0.0); m1.mass=10.0;  m1.retain=800
        m2.pos=vector(-10,0,3); m2.vel=vector(-0.3,-1.0,0.5);m2.mass=10.0; m2.retain=800
        m3.pos=vector( 0,0,0);  m3.vel=vector( 0, 0.1,0.1);  m3.mass=10.0; m3.retain=800
        scene.range=30
        resetscene(m1,m2,m3)

scene.bind('keydown', keyInput)

## initiate with the simplest system
m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
m2.pos=vector( 0,0,0);   m2.vel=vector( 0, 0,0); m2.mass=10
m3.pos=vector( 10,0,0);  m3.vel=vector(0,-1,0);  m3.mass=0.1
scene.range=20
resetscene(m1,m2,m3)
threebody = False;

dt = 0.1 ## the time step.  Keep this as small as possible.


while True:
    rate(200)
    y = [ m1.pos, m1.vel, m2.pos, m2.vel, m3.pos, m3.vel] 
    k1 = dydt(y)
    k2 = dydt(add(y, mult( 0.5, k1)))
    k3 = dydt(add(y, mult( 0.5, k2))) 
    k4 = dydt(add(y,k3))
    dy = add(add( mult( (1.0/6.0), k1), mult( (1.0/3.0), k2)),add( mult( (1.0/3.0), k3) , mult( (1.0/6.0), k4)))
    if threebody:    
        m1.pos += dy[0]; m1.vel += dy[1]
    if mag(m2.pos-m3.pos)>1:
        m2.pos += dy[2]; m2.vel += dy[3]
        m3.pos += dy[4]; m3.vel += dy[5]
